package lineaHorizontal;

import generalidades.Utilidades;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase que almacena mtodos tiles para problemas de etiquetado sobre lnea
 * horizontal.
 * 
 * @author Jos Manuel Ponce
 */
public class UtilidadesHorizontal
{
	/**
	 * Mtodo que refleja en la consola el estado grfico de las etiquetas.
	 * 
	 * @param longitudLinea
	 *            int Longitud de la lnea horizontal
	 * @param puntos
	 *            List<Integer> Lista de puntos
	 * @param lineaSuperior
	 *            List<Boolean> Lnea de etiquetas superior
	 * @param lineaInferior
	 *            List<Boolean> Lnea de etiquetas inferior
	 */
	public static void representar(int longitudLinea, List<Integer> puntos,
			List<Integer> lineaSuperior, List<Integer> lineaInferior)
	{
		int i;
		
		// Separador
		System.out.println();
		for (i = 0; i <= longitudLinea; i++)
		{
			System.out.print("#");
		}
		System.out.println();
		
		// ndices
		for (i = 0; i <= longitudLinea; i++)
		{
			System.out.print(Utilidades.mod10(i));
		}
		System.out.println();
		
		// Lnea superior de etiquetas
		for (i = 0; i <= longitudLinea; i++)
		{
			if (null != lineaSuperior.get(i))
			{
				System.out.print(lineaSuperior.get(i));
			} else
			{
				System.out.print(" ");
			}
		}
		System.out.println();
		
		// Representacin de la lnea horizontal con los puntos a etiquetar
		int posicion = 0;
		int punto = -1;
		int anterior = -1;
		for (i = 0; i < puntos.size(); i++)
		{
			punto = puntos.get(i);
			while ((posicion <= longitudLinea) && (posicion < punto))
			{
				System.out.print("-");
				posicion++;
			}
			if (punto != anterior)
			{
				System.out.print(":");
				posicion++;
			}
			anterior = punto;
		}
		posicion = punto+1;
		while (posicion <= longitudLinea)
		{
			System.out.print("-");
			posicion++;
		}
		System.out.println();
		
		// Lnea inferior de etiquetas
		for (i = 0; i <= longitudLinea; i++)
		{
			if (null != lineaInferior.get(i))
			{
				System.out.print(lineaInferior.get(i));
			} else
			{
				System.out.print(" ");
			}
		}
		System.out.println();
		
		// ndices
		for (i = 0; i <= longitudLinea; i++)
		{
			System.out.print(Utilidades.mod10(i));
		}
		System.out.println();
		
		// Separador
		for (i = 0; i <= longitudLinea; i++)
		{
			System.out.print("#");
		}
		System.out.println();
		System.out.println();
	}
	
	/**
	 * Mtodo que devuelve cierto si la primera sombra es menor que la segunda.
	 * 
	 * @param s1
	 *            Primera sombra
	 * @param s2
	 *            Segunda sombra
	 * @return boolean Cierto si la primera sombra es menor que la segunda
	 */
	public static boolean sombraMenorIgualQue(SombraHorizontal s1,
			SombraHorizontal s2)
	{
		boolean dev = false;
		if (((s1.getTk() <= s2.getTk()) && (s1.getBk() <= s2.getBk()))
				|| ((s1.getTk() <= s2.getBk()) && (s1.getBk() <= s2.getTk())))
		{
			dev = true;
		}
		return dev;
	}
	
	/**
	 * Mtodo que calcula cul de las 3 sombras indicadas es la menor.
	 * 
	 * @param s1
	 *            Primera sombra
	 * @param s2
	 *            Segunda sombra
	 * @param s3
	 *            Tercera sombra
	 * @return Posicin de la sombra menor. Devuelve -1 si no hay ninguna sombra
	 *         vlida.
	 */
	public static int hallarSombraMenor3(SombraHorizontal s1,
			SombraHorizontal s2, SombraHorizontal s3)
	{
		int dev = -1;
		if (sombraMenorIgualQue(s1, s2))
		{
			if (sombraMenorIgualQue(s1, s3))
			{
				dev = 1;
			} else
			{
				dev = 3;
			}
		} else
		{
			if (sombraMenorIgualQue(s2, s3))
			{
				dev = 2;
			} else
			{
				dev = 3;
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que calcula cul de las 4 sombras indicadas es la menor.
	 * 
	 * @param s1
	 *            Primera sombra
	 * @param s2
	 *            Segunda sombra
	 * @param s3
	 *            Tercera sombra
	 * @param s4
	 *            Cuarta sombra
	 * @return Posicin de la sombra menor. Devuelve -1 si no hay ninguna sombra
	 *         vlida.
	 */
	public static int hallarSombraMenor4(SombraHorizontal s1,
			SombraHorizontal s2, SombraHorizontal s3, SombraHorizontal s4)
	{
		int dev = -1;
		int menor;
		int indice;
		List<SombraHorizontal> sombras = new ArrayList<SombraHorizontal>();
		List<Boolean> sombrasValidas = new ArrayList<Boolean>();
		for (int i = 0; i < 4; i++)
		{
			sombrasValidas.add(false);
		}
		// Rellenamos la lista con sombras != null
		if (null != s1)
		{
			sombras.add(s1);
			sombrasValidas.set(0, true);
		}
		if (null != s2)
		{
			sombras.add(s2);
			sombrasValidas.set(1, true);
		}
		if (null != s3)
		{
			sombras.add(s3);
			sombrasValidas.set(2, true);
		}
		if (null != s4)
		{
			sombras.add(s4);
			sombrasValidas.set(3, true);
		}
		// Hay 4 sombras vlidas
		if (4 == sombras.size())
		{
			System.out.println("Hay 4 sombras vlidas:");
			if (UtilidadesHorizontal.sombraMenorIgualQue(s1, s2)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s1, s3)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s1, s4))
			{
				dev = 1;
			} else if (UtilidadesHorizontal.sombraMenorIgualQue(s2, s1)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s2, s3)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s2, s4))
			{
				dev = 2;
			} else if (UtilidadesHorizontal.sombraMenorIgualQue(s3, s1)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s3, s2)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s3, s4))
			{
				dev = 3;
			} else if (UtilidadesHorizontal.sombraMenorIgualQue(s4, s1)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s4, s2)
					&& UtilidadesHorizontal.sombraMenorIgualQue(s4, s3))
			{
				dev = 4;
			}
		} else if (3 == sombras.size()) // Hay 3 sombras vlidas
		{
			System.out.println("Hay 3 sombras vlidas:");
			menor = hallarSombraMenor3(sombras.get(0), sombras.get(1), sombras
					.get(2));
			indice = 1;
			for (int i = 0; i < 4; i++)
			{
				if (sombrasValidas.get(i))
				{
					if (indice == menor)
					{
						dev = i + 1;
						break;
					}
					indice++;
				}
			}
		} else if (2 == sombras.size()) // Hay 2 sombras vlidas
		{
			System.out.println("Hay 2 sombras vlidas:");
			if (sombraMenorIgualQue(sombras.get(0), sombras.get(1)))
			{
				menor = 1;
			} else
			{
				menor = 2;
			}
			indice = 1;
			for (int i = 0; i < 4; i++)
			{
				if (sombrasValidas.get(i))
				{
					if (indice == menor)
					{
						dev = i + 1;
						break;
					}
					i++;
				}
			}
		} else if (1 == sombras.size()) // Hay slo 1 sombra vlida
		{
			System.out.println("Hay 1 sombras vlidas:");
			for (int i = 0; i < 4; i++)
			{
				if (sombrasValidas.get(i))
				{
					dev = i + 1;
				}
			}
		}
		else if(0==sombras.size()) // No hay ninguna sombra vlida
		{
			System.out.println("No hay sombras vlidas:");
		}
		return dev;
	}
}
