package lineaHorizontal.h_4s_1c;

import generalidades.ConjuntoListas;
import generalidades.Constantes;
import generalidades.Utilidades;

import java.util.ArrayList;
import java.util.List;

import lineaHorizontal.UtilidadesHorizontal;

/**
 * Clase que contiene los mtodos necesarios para solucionar el problema
 * H-4S-1C.
 * 
 * @author Jos Manuel Ponce
 */
public class Solucion_H_4S_1C
{
	/**
	 * Lista de puntos.
	 */
	private List<Integer> puntos;
	
	/**
	 * Lista con los tamaos (cuadrados) de las etiquetas.
	 */
	private List<Integer> etiquetas;
	
	/**
	 * Lista de posiciones finales de las etiquetas respecto a los puntos.
	 */
	private List<String> resultado;
	
	/**
	 * Lista auxiliar cuyos elementos denotan si estn ocupadas por una etiqueta
	 * las posiciones que representan por encima de la lnea de entrada.
	 */
	private List<Integer> lineaSuperior;
	
	/**
	 * Lista auxiliar cuyos elementos denotan si estn ocupadas por una etiqueta
	 * las posiciones que representan por debajo de la lnea de entrada.
	 */
	private List<Integer> lineaInferior;
	
	/**
	 * Nmero de pares punto-etiqueta.
	 */
	private int numElementos;
	
	/**
	 * Nmero de divisiones de la lnea de entrada.
	 */
	private int longitudLinea;
	
	/**
	 * Constructor sin parmetros.
	 */
	public Solucion_H_4S_1C()
	{
		super();
	}
	
	/**
	 * Constructor con parmetros.
	 * 
	 * @param puntos
	 *           List<Integer> Lista de puntos
	 * @param etiquetas
	 *           Integer Lista de tamaos (cuadrados) de las etiquetas
	 */
	public Solucion_H_4S_1C(List<Integer> puntos, List<Integer> etiquetas)
	{
		this.puntos = puntos;
		this.etiquetas = etiquetas;
		this.numElementos = puntos.size();
		this.resultado = new ArrayList<String>();
		this.lineaSuperior = new ArrayList<Integer>();
		this.lineaInferior = new ArrayList<Integer>();
	}
	
	/**
	 * Mtodo que imprime la solucin al problema, si la hubiese.
	 */
	public void obtenerSolucion()
	{
		ConjuntoListas conjunto;
		boolean haySolucion = false;
		
		if (Utilidades.valoresValidos(this.puntos)
				&& Utilidades.valoresValidos(this.etiquetas))
		{
			if (!Utilidades.puntosOrdenados(puntos))
			{
				conjunto = Utilidades.ordenarListasInteger(this.puntos,
						this.etiquetas);
				this.puntos = conjunto.getListaPuntos();
				this.etiquetas = conjunto.getListaEtiquetas();
				System.out
						.println("La lista de puntos no est ordenada. Procedemos a ordenarla.");
				System.out
						.println("Los pares (punto , etiqueta) indicados han sido:");
				for (int i = 0; i < numElementos; i++)
				{
					System.out.print("par " + (i + 1) + ": (" + this.puntos.get(i)
							+ " , " + this.etiquetas.get(i) + ")\n");
				}
			} else
			{
				System.out
						.println("La lista de puntos est ordenada. Procedemos a la resolucin del problema:");
			}
			inicializaValores();
			
			int desp;
			int indice = 1;
			for (int i = 0; i < this.numElementos; i++)
			{
				haySolucion = false;
				for (desp = 0; (desp <= (this.etiquetas.get(i) + 1))
						&& !haySolucion; desp++)
				{
					boolean etiquetado = false;
					if (!etiquetado
							&& probarEtiquetaArriba(this.puntos.get(i), this.etiquetas
									.get(i), desp))
					{
						haySolucion = true;
						etiquetado = true;
						aplicarEtiquetaArriba(this.puntos.get(i), this.etiquetas
								.get(i), desp, indice);
						indice++;
						if (0 == desp)
						{
							this.resultado.set(i, Constantes.POS_ARRIBA_IZQUIERDA);
						} else if ((this.etiquetas.get(i) + 1) == desp)
						{
							this.resultado.set(i, Constantes.POS_ARRIBA_DERECHA);
						} else
						{
							this.resultado.set(i, Constantes.POS_ARRIBA);
						}
					} else if (!etiquetado
							&& probarEtiquetaAbajo(this.puntos.get(i), this.etiquetas
									.get(i), desp))
					{
						haySolucion = true;
						etiquetado = true;
						aplicarEtiquetaAbajo(this.puntos.get(i), this.etiquetas
								.get(i), desp, indice);
						indice++;
						if (0 == desp)
						{
							this.resultado.set(i, Constantes.POS_ABAJO_IZQUIERDA);
						} else if ((this.etiquetas.get(i) + 1) == desp)
						{
							this.resultado.set(i, Constantes.POS_ABAJO_DERECHA);
						} else
						{
							this.resultado.set(i, Constantes.POS_ABAJO);
						}
					}
				}
				if (!haySolucion)
				{
					break;
				}
			}
			
			System.out.println("Se han obtenido las siguientes posiciones:");
			Utilidades.imprimeListaString(resultado);
			if (!haySolucion)
			{
				System.out
						.println("No se ha podido encontrar una solucin vlida a ");
				System.out
						.println("este problema con los datos que ha proporcionado.");
				System.out
						.println("Grficamente, la solucin parcial obtenida ha sido la siguiente:");
			}
			if (haySolucion)
			{
				System.out
						.println("Grficamente, la solucin del problema se puede representar como sigue:");
			}
			UtilidadesHorizontal.representar(this.longitudLinea, this.puntos,
					this.lineaSuperior, this.lineaInferior);
			System.out.println(">> Fin del problema.");
		} else
		{
			System.out
					.println("Los valores de los puntos y etiquetas han de ser mayores que cero.");
			System.out.println("Vuelva a ejecutar el programa si lo desea.");
		}
	}
	
	/**
	 * Mtodo que inicializa las listas "resultado", "lineaSuperior" y
	 * "lineaInferior".
	 */
	public void inicializaValores()
	{
		this.longitudLinea = this.puntos.get(numElementos - 1) + 1
				+ Utilidades.maxValorLista(this.etiquetas);
		for (int i = 0; i <= this.longitudLinea; i++)
		{
			this.lineaSuperior.add(i, null);
			this.lineaInferior.add(i, null);
		}
		for (int i = 0; i < this.numElementos; i++)
		{
			this.resultado.add(i, "--");
		}
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado por
	 * el desplazamiento en la lnea de arriba.
	 * 
	 * @param punto
	 *           Punto a etiquetar
	 * @param etiqueta
	 *           Tamao de la etiqueta que se va a aadir
	 * @param desp
	 *           Desplazamiento
	 * @return boolean Cierto si se puede aadir la etiqueta
	 */
	public boolean probarEtiquetaArriba(Integer punto, Integer etiqueta,
			Integer desp)
	{
		boolean dev = true;
		for (int i = punto - 1 + desp; (i >= 0) && (i >= punto - etiqueta + desp)
				&& dev; i--)
		{
			if (null != this.lineaSuperior.get(i))
			{
				dev = false;
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado por
	 * el desplazamiento en la lnea de abajo.
	 * 
	 * @param punto
	 *           Punto a etiquetar
	 * @param etiqueta
	 *           Tamao de la etiqueta que se va a aadir
	 * @param desp
	 *           Desplazamiento
	 * @return boolean Cierto si se puede aadir la etiqueta
	 */
	public boolean probarEtiquetaAbajo(Integer punto, Integer etiqueta,
			Integer desp)
	{
		boolean dev = true;
		int i;
		for (i = punto - 1 + desp; (i >= 0) && (i >= punto - etiqueta + desp)
				&& dev; i--)
		{
			if (null != this.lineaInferior.get(i))
			{
				dev = false;
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado por el
	 * desplazamiento en la lnea de arriba.
	 * 
	 * @param punto
	 *           Punto a etiquetar
	 * @param etiqueta
	 *           Tamao de la etiqueta que se va a aadir
	 * @param desp
	 *           Desplazamiento
	 * @param indice
	 *           ndice aplicado a la etiqueta que se aade
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaArriba(Integer punto, Integer etiqueta,
			Integer desp, int indice)
	{
		boolean dev = true;
		indice = indice % 10;
		int i;
		for (i = punto - 1 + desp; (i >= 0) && (i >= punto - etiqueta + desp)
				&& dev; i--)
		{
			this.lineaSuperior.set(i, indice);
		}
		return dev;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado por el
	 * desplazamiento en la lnea de arriba.
	 * 
	 * @param punto
	 *           Punto a etiquetar
	 * @param etiqueta
	 *           Tamao de la etiqueta que se va a aadir
	 * @param desp
	 *           Desplazamiento
	 * @param indice
	 *           ndice aplicado a la etiqueta que se aade
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaAbajo(Integer punto, Integer etiqueta,
			Integer desp, int indice)
	{
		boolean dev = true;
		indice = indice % 10;
		int i;
		for (i = punto - 1 + desp; (i >= 0) && (i >= punto - etiqueta + desp)
				&& dev; i--)
		{
			this.lineaInferior.set(i, indice);
		}
		return dev;
	}
}
