package lineaOblicua.oc_4s;

import generalidades.ConjuntoListas;
import generalidades.Constantes;
import generalidades.Utilidades;

import java.util.ArrayList;
import java.util.List;

import lineaOblicua.UtilidadesOblicua;

/**
 * Clase que contiene los mtodos necesarios para solucionar el problema OC-4S.
 * 
 * @author Jos Manuel Ponce
 */
public class Solucion_OC_4S
{
	/**
	 * Lista de puntos.
	 */
	private List<Integer> puntos;
	
	/**
	 * Lista de etiquetas.
	 */
	private List<Integer> etiquetas;
	
	/**
	 * Lista de posiciones finales de las etiquetas respecto a los puntos.
	 */
	private List<String> resultado;
	
	/**
	 * Nmero de pares punto-etiqueta.
	 */
	private int numElementos;
	
	/**
	 * Nmero de divisiones de la lnea de entrada.
	 */
	private int longitudLinea;
	
	/**
	 * Objeto que almacena la posicin de las etiquetas que ya se han aadido.
	 */
	private boolean[][] espacioEtiquetas;
	
	/**
	 * Constructor sin parmetros.
	 */
	public Solucion_OC_4S()
	{
		super();
	}
	
	/**
	 * Constructor con parmetros.
	 * 
	 * @param puntos
	 *            List<Integer> Lista de puntos
	 * @param etiquetas
	 *            List<Integer> Lista de etiquetas
	 */
	public Solucion_OC_4S(List<Integer> puntos, List<Integer> etiquetas)
	{
		this.puntos = puntos;
		this.etiquetas = etiquetas;
		this.numElementos = puntos.size();
		this.resultado = new ArrayList<String>();
	}
	
	/**
	 * Mtodo que imprime la solucin al problema, si la hubiese.
	 */
	public void obtenerSolucion()
	{
		boolean haySolucion = true;
		boolean puntoEtiquetado;
		
		if ((Utilidades.valoresValidos(this.puntos))
				&& (Utilidades.valoresValidos(this.etiquetas)))
		{
			if (!Utilidades.puntosOrdenados(puntos))
			{
				ConjuntoListas conjunto = Utilidades.ordenarListasInteger(
						this.puntos, this.etiquetas);
				this.puntos = conjunto.getListaPuntos();
				this.etiquetas = conjunto.getListaEtiquetas();
				System.out
						.println("La lista de puntos no est ordenada. Procedemos a ordenarla.");
				System.out
						.println("Los pares (punto , etiqueta) indicados han sido:");
				for (int i = 0; i < numElementos; i++)
				{
					System.out.print("par " + (i + 1) + ": ("
							+ this.puntos.get(i) + " , "
							+ this.etiquetas.get(i) + ")\n");
				}
			} else
			{
				System.out
						.println("La lista de puntos est ordenada. Procedemos a la resolucin del problema:");
			}
			inicializaValores();
			
			for (int i = 0; (i < this.numElementos) && haySolucion; i++)
			{
				puntoEtiquetado = false;
				
				// Etiquetado en la posicin ABAJO-IZQUIERDA
				if (!puntoEtiquetado
						&& this.probarEtiquetaBL(this.puntos.get(i),
								this.etiquetas.get(i)))
				{
					puntoEtiquetado = true;
					this.aplicarEtiquetaBL(this.puntos.get(i), this.etiquetas
							.get(i));
					this.resultado.set(i, Constantes.POS_ABAJO_IZQUIERDA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				}
				
				// Etiquetado en la posicin IZQUIERDA
				if (!puntoEtiquetado)
				{
					int desliza = this.probarEtiquetaLL(this.puntos.get(i),
							this.etiquetas.get(i));
					if (-1 != desliza)
					{
						puntoEtiquetado = true;
						this.aplicarEtiquetaLL(this.puntos.get(i),
								this.etiquetas.get(i), desliza);
						this.resultado.set(i, Constantes.POS_IZQUIERDA);
						System.out
								.println("\nEstado del espacio despus de etiquetar el punto"
										+ (this.puntos.get(i)));
						UtilidadesOblicua
								.representarEspacio(this.espacioEtiquetas);
						if (-1 != desliza)
						{
							System.out.println("Grado de deslizamiento: "
									+ desliza);
						}
					}
				}
				
				// Etiquetado en la posicin ABAJO
				if (!puntoEtiquetado)
				{
					int desliza = this.probarEtiquetaBB(this.puntos.get(i),
							this.etiquetas.get(i));
					if (-1 != desliza)
					{
						puntoEtiquetado = true;
						this.aplicarEtiquetaBB(this.puntos.get(i),
								this.etiquetas.get(i), desliza);
						this.resultado.set(i, Constantes.POS_ABAJO);
						System.out
								.println("\nEstado del espacio despus de etiquetar el punto"
										+ (this.puntos.get(i)));
						UtilidadesOblicua
								.representarEspacio(this.espacioEtiquetas);
						if (-1 != desliza)
						{
							System.out.println("Grado de deslizamiento: "
									+ desliza);
						}
					}
				}
				
				// Etiquetado en la posicin ARRIBA-IZQUIERDA
				if (!puntoEtiquetado
						&& this.probarEtiquetaTL(this.puntos.get(i),
								this.etiquetas.get(i)))
				{
					puntoEtiquetado = true;
					this.aplicarEtiquetaTL(this.puntos.get(i), this.etiquetas
							.get(i));
					this.resultado.set(i, Constantes.POS_ARRIBA_IZQUIERDA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				}
				
				// Etiquetado en la posicin ABAJO-DERECHA
				if (!puntoEtiquetado
						&& this.probarEtiquetaBR(this.puntos.get(i),
								this.etiquetas.get(i)))
				{
					puntoEtiquetado = true;
					this.aplicarEtiquetaBR(this.puntos.get(i), this.etiquetas
							.get(i));
					this.resultado.set(i, Constantes.POS_ABAJO_DERECHA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				}
				
				// Etiquetado en la posicin ARRIBA
				if (!puntoEtiquetado)
				{
					int desliza = this.probarEtiquetaTT(this.puntos.get(i),
							this.etiquetas.get(i));
					if (-1 != desliza)
					{
						puntoEtiquetado = true;
						this.aplicarEtiquetaTT(this.puntos.get(i),
								this.etiquetas.get(i), desliza);
						this.resultado.set(i, Constantes.POS_ARRIBA);
						System.out
								.println("\nEstado del espacio despus de etiquetar el punto"
										+ (this.puntos.get(i)));
						UtilidadesOblicua
								.representarEspacio(this.espacioEtiquetas);
						if (-1 != desliza)
						{
							System.out.println("Grado de deslizamiento: "
									+ desliza);
						}
					}
				}
				
				// Etiquetado en la posicin DERECHA
				if (!puntoEtiquetado)
				{
					int desliza = this.probarEtiquetaRR(this.puntos.get(i),
							this.etiquetas.get(i));
					if (-1 != desliza)
					{
						puntoEtiquetado = true;
						this.aplicarEtiquetaRR(this.puntos.get(i),
								this.etiquetas.get(i), desliza);
						this.resultado.set(i, Constantes.POS_DERECHA);
						System.out
								.println("\nEstado del espacio despus de etiquetar el punto"
										+ (this.puntos.get(i)));
						UtilidadesOblicua
								.representarEspacio(this.espacioEtiquetas);
						if (-1 != desliza)
						{
							System.out.println("Grado de deslizamiento: "
									+ desliza);
						}
					}
				}
				
				// Etiquetado en la posicin ARRIBA-DERECHA
				if (!puntoEtiquetado
						&& this.probarEtiquetaTR(this.puntos.get(i),
								this.etiquetas.get(i)))
				{
					puntoEtiquetado = true;
					this.aplicarEtiquetaTR(this.puntos.get(i), this.etiquetas
							.get(i));
					this.resultado.set(i, Constantes.POS_ABAJO_DERECHA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				}
				
				if (!puntoEtiquetado)
				{
					haySolucion = false;
				}
			}
			
			System.out.println("\nSe han obtenido las siguientes posiciones:");
			Utilidades.imprimeListaString(resultado);
			if (!haySolucion)
			{
				System.out
						.println("No se ha podido encontrar una solucin vlida a ");
				System.out
						.println("este problema con los datos que ha proporcionado.");
				System.out
						.println("Grficamente, la solucin parcial obtenida ha sido la siguiente:");
			}
			if (haySolucion)
			{
				System.out
						.println("\nGrficamente, la solucin del problema se puede representar como sigue:");
			}
			UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
			System.out.println(">> Fin del problema.");
		} else
		{
			System.out
					.println("Los valores de los puntos y etiquetas han de ser mayores que cero.");
			System.out.println("Vuelva a ejecutar el programa si lo desea.");
		}
	}
	
	/**
	 * Mtodo que inicializa las listas "resultado", "lineaSuperior" y
	 * "lineaInferior".
	 */
	public void inicializaValores()
	{
		this.longitudLinea = this.puntos.get(numElementos - 1) + 2
				+ Utilidades.maxValorLista(this.etiquetas);
		this.espacioEtiquetas = new boolean[this.longitudLinea][this.longitudLinea];
		for (int i = 0; i < this.longitudLinea; i++)
		{
			for (int j = 0; j < this.longitudLinea; j++)
			{
				this.espacioEtiquetas[i][j] = false;
			}
		}
		for (int i = 0; i < this.numElementos; i++)
		{
			this.resultado.add(i, "--");
		}
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin TL para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaTL(Integer punto, Integer etiqueta)
	{
		boolean dev = true;
		int x;
		int y = punto;
		for (int desp = 0; (desp < etiqueta) && (y + desp < this.longitudLinea)
				&& dev; desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - etiqueta) && dev; x--)
			{
				if (this.espacioEtiquetas[y + desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin BL para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaBL(Integer punto, Integer etiqueta)
	{
		boolean dev = true;
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < etiqueta) && (y - desp >= 0) && dev; desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - etiqueta) && dev; x--)
			{
				if (this.espacioEtiquetas[y - desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin TR para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaTR(Integer punto, Integer etiqueta)
	{
		boolean dev = true;
		int x;
		int y = punto;
		for (int desp = 0; (desp < etiqueta) && (y + desp < this.longitudLinea)
				&& dev; desp++)
		{
			for (x = punto; (x < this.longitudLinea) && (x < punto + etiqueta)
					&& dev; x++)
			{
				if (this.espacioEtiquetas[y + desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin BR para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaBR(Integer punto, Integer etiqueta)
	{
		boolean dev = true;
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < etiqueta) && (y - desp >= 0) && dev; desp++)
		{
			for (x = punto; (x < this.longitudLinea) && (x < punto + etiqueta)
					&& dev; x++)
			{
				if (this.espacioEtiquetas[y - desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin LL para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return int Deslizamiento. -1 Si no se puede etiquetar
	 */
	public int probarEtiquetaLL(Integer punto, Integer etiqueta)
	{
		boolean fin = false;
		boolean sePuedeEtiquetar;
		int desliza = -1;
		int x;
		for (int deslizamiento = 1; (deslizamiento < etiqueta) && !fin; deslizamiento++)
		{
			sePuedeEtiquetar = true;
			int y = punto - 1 + deslizamiento;
			for (int desp = 0; (desp < etiqueta) && (y - desp >= 0)
					&& sePuedeEtiquetar && !fin; desp++)
			{
				for (x = punto - 1; (x >= 0) && (x >= punto - etiqueta)
						&& sePuedeEtiquetar; x--)
				{
					if (this.espacioEtiquetas[y - desp][x])
					{
						sePuedeEtiquetar = false;
					}
				}
			}
			if (sePuedeEtiquetar)
			{
				desliza = deslizamiento;
				fin = true;
			}
		}
		return desliza;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin BB para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return int Deslizamiento. -1 Si no se puede etiquetar
	 */
	public int probarEtiquetaBB(Integer punto, Integer etiqueta)
	{
		boolean fin = false;
		boolean sePuedeEtiquetar;
		int desliza = -1;
		int x;
		for (int deslizamiento = 1; (deslizamiento < etiqueta) && !fin; deslizamiento++)
		{
			sePuedeEtiquetar = true;
			int y = punto - 1;
			for (int desp = 0; (desp < etiqueta) && (y - desp >= 0)
					&& sePuedeEtiquetar && !fin; desp++)
			{
				for (x = punto - 1 + deslizamiento; (x >= 0)
						&& (x >= punto - etiqueta + deslizamiento)
						&& sePuedeEtiquetar; x--)
				{
					if (this.espacioEtiquetas[y - desp][x])
					{
						sePuedeEtiquetar = false;
					}
				}
			}
			if (sePuedeEtiquetar)
			{
				desliza = deslizamiento;
				fin = true;
			}
		}
		return desliza;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin TT para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return int Deslizamiento. -1 Si no se puede etiquetar
	 */
	public int probarEtiquetaTT(Integer punto, Integer etiqueta)
	{
		boolean fin = false;
		boolean sePuedeEtiquetar;
		int desliza = -1;
		int x;
		for (int deslizamiento = 1; (deslizamiento < etiqueta) && !fin; deslizamiento++)
		{
			sePuedeEtiquetar = true;
			int y = punto;
			for (int desp = 0; (desp < etiqueta)
					&& (y + desp < this.longitudLinea) && sePuedeEtiquetar
					&& !fin; desp++)
			{
				for (x = punto - 1 + deslizamiento; (x >= 0)
						&& (x >= punto - etiqueta + deslizamiento)
						&& sePuedeEtiquetar; x--)
				{
					if (this.espacioEtiquetas[y + desp][x])
					{
						sePuedeEtiquetar = false;
					}
				}
			}
			if (sePuedeEtiquetar)
			{
				desliza = deslizamiento;
				fin = true;
			}
		}
		return desliza;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin RR para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return int Deslizamiento. -1 Si no se puede etiquetar
	 */
	public int probarEtiquetaRR(Integer punto, Integer etiqueta)
	{
		boolean fin = false;
		boolean sePuedeEtiquetar;
		int desliza = -1;
		int x;
		for (int deslizamiento = 1; (deslizamiento < etiqueta) && !fin; deslizamiento++)
		{
			sePuedeEtiquetar = true;
			int y = punto - 1 + deslizamiento;
			for (int desp = 0; (desp < etiqueta) && (y - desp >= 0)
					&& sePuedeEtiquetar && !fin; desp++)
			{
				for (x = punto; (x < this.longitudLinea)
						&& (x < punto + etiqueta) && sePuedeEtiquetar; x++)
				{
					if (this.espacioEtiquetas[y - desp][x])
					{
						sePuedeEtiquetar = false;
					}
				}
			}
			if (sePuedeEtiquetar)
			{
				desliza = deslizamiento;
				fin = true;
			}
		}
		return desliza;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin TL.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 */
	public void aplicarEtiquetaTL(Integer punto, Integer etiqueta)
	{
		int x;
		int y = punto;
		for (int desp = 0; (desp < etiqueta) && (y + desp < this.longitudLinea); desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - etiqueta); x--)
			{
				this.espacioEtiquetas[y + desp][x] = true;
			}
		}
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin BL.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 */
	public void aplicarEtiquetaBL(Integer punto, Integer etiqueta)
	{
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < etiqueta) && (y - desp >= 0); desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - etiqueta); x--)
			{
				this.espacioEtiquetas[y - desp][x] = true;
			}
		}
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin TR.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 */
	public void aplicarEtiquetaTR(Integer punto, Integer etiqueta)
	{
		int x;
		int y = punto;
		for (int desp = 0; (desp < etiqueta) && (y + desp < this.longitudLinea); desp++)
		{
			for (x = punto; (x < this.longitudLinea) && (x < punto + etiqueta); x++)
			{
				this.espacioEtiquetas[y + desp][x] = true;
			}
		}
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin BR.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 */
	public void aplicarEtiquetaBR(Integer punto, Integer etiqueta)
	{
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < etiqueta) && (y - desp >= 0); desp++)
		{
			for (x = punto; (x < this.longitudLinea) && (x < punto + etiqueta); x++)
			{
				this.espacioEtiquetas[y - desp][x] = true;
			}
		}
	}
	
	/**
	 * Mtodo que agrega la etiqueta en la posicin LL y pasa al siguiente
	 * punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param delizamiento
	 *            Deslizamiento
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaLL(Integer punto, Integer etiqueta,
			Integer deslizamiento)
	{
		boolean dev = true;
		int x;
		int y = punto - 1 + deslizamiento;
		for (int desp = 0; (desp < etiqueta) && (y - desp >= 0); desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - etiqueta); x--)
			{
				this.espacioEtiquetas[y - desp][x] = true;
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que agrega la etiqueta en la posicin BB y pasa al siguiente
	 * punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param delizamiento
	 *            Deslizamiento
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaBB(Integer punto, Integer etiqueta,
			Integer deslizamiento)
	{
		boolean dev = true;
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < etiqueta) && (y - desp >= 0); desp++)
		{
			for (x = punto - 1 + deslizamiento; (x >= 0)
					&& (x >= punto - etiqueta + deslizamiento); x--)
			{
				this.espacioEtiquetas[y - desp][x] = true;
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que agrega la etiqueta en la posicin TT y pasa al siguiente
	 * punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param delizamiento
	 *            Deslizamiento
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaTT(Integer punto, Integer etiqueta,
			Integer deslizamiento)
	{
		boolean dev = true;
		int x;
		int y = punto;
		for (int desp = 0; (desp < etiqueta) && (y + desp < this.longitudLinea); desp++)
		{
			for (x = punto - 1 + deslizamiento; (x >= 0)
					&& (x >= punto - etiqueta + deslizamiento); x--)
			{
				this.espacioEtiquetas[y + desp][x] = true;
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que agrega la etiqueta en la posicin RR y pasa al siguiente
	 * punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param delizamiento
	 *            Deslizamiento
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaRR(Integer punto, Integer etiqueta,
			Integer deslizamiento)
	{
		boolean dev = true;
		int x;
		int y = punto - 1 + deslizamiento;
		for (int desp = 0; (desp < etiqueta) && (y - desp >= 0); desp++)
		{
			for (x = punto; (x < this.longitudLinea) && (x < punto + etiqueta); x++)
			{
				this.espacioEtiquetas[y - desp][x] = true;
			}
		}
		return dev;
	}
}