package lineaOblicua.orm_4p;

import generalidades.Constantes;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Clase principal que resuelve el problema ORM-4P.
 * 
 * @author Jos Manuel Ponce
 */
public class Main_ORM_4P
{
	/**
	 * Instancia de un objeto Solucion_OR1_4P.
	 */
	private static Solucion_ORM_4P solucion;
	
	/**
	 * Cdigo a ejecutar.
	 * 
	 * @param args
	 *            Argumentos
	 */
	public static void main(String[] args)
	{
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		System.out.println(Constantes.CABECERA_ORM_4P);
		System.out.println();
		
		boolean salir = false, error = false, finProblema = false;
		int i = 0, numElementos = 0;
		String cadenaEntrada = "";
		List<Integer> puntos = new ArrayList<Integer>();
		List<Integer> alturas = new ArrayList<Integer>();
		List<Integer> anchuras = new ArrayList<Integer>();
		
		try
		{
			while (!salir && !finProblema)
			{
				System.out
						.println("Pulse la tecla \"T\" para introducir los datos desde el teclado.");
				System.out
						.println("Pulse la tecla \"F\" si desea que la aplicacin obtenga los datos a partir de un fichero de texto.");
				System.out
						.println("Pulse la tecla \"I\" si desea informacin adicional acerca de este problema.");
				System.out
						.println("Pulse la tecla \"S\" para salir de la aplicacin.");
				
				cadenaEntrada = in.readLine();
				
				if ("I".equals(cadenaEntrada) || "i".equals(cadenaEntrada))
				{
					System.out.println(Constantes.INFO_ORM_4P);
				}

				else
				{
					if ("T".equals(cadenaEntrada) || "t".equals(cadenaEntrada))
					{
						// El usuario prefiere introducir los datos desde el
						// teclado.
						System.out
								.println("Introduzca el nmero de elementos. Pulse \"S\" para salir.");
						cadenaEntrada = in.readLine();
						
						if (cadenaEntrada.equals("S")
								|| cadenaEntrada.equals("s"))
						{
							System.out
									.println("La aplicacin ha finalizado correctamente.");
							salir = true;
							break;
						} else
						{
							numElementos = Integer.valueOf(cadenaEntrada)
									.intValue();
							if (1 > numElementos)
							{
								System.out
										.println("El nmero de elementos debe ser mayor que cero.");
								salir = true;
								break;
							} else
							{
								puntos = new ArrayList<Integer>(numElementos);
								alturas = new ArrayList<Integer>(numElementos);
								anchuras = new ArrayList<Integer>(numElementos);
								System.out.println("Ha indicado "
										+ numElementos + " elementos. ");
								i = 1;
								while (i <= numElementos)
								{
									System.out.println("Introduzca el punto "
											+ i + ":");
									puntos.add(Integer.valueOf(in.readLine())
											.intValue());
									i++;
								}
								i = 1;
								while (i <= numElementos)
								{
									System.out
											.println("Introduzca la altura para el punto "
													+ i + ":");
									alturas.add(Integer.valueOf(in.readLine())
											.intValue());
									i++;
								}
								i = 1;
								while (i <= numElementos)
								{
									System.out
											.println("Introduzca la anchura para el punto "
													+ i + ":");
									anchuras.add(Integer.valueOf(in.readLine())
											.intValue());
									i++;
								}
								finProblema = true;
							}
						}
					}

					else if (cadenaEntrada.equals("F")
							|| cadenaEntrada.equals("f"))
					{
						// El usuario prefiere utilizar un fichero de texto para
						// indicar los datos.
						System.out
								.println("Guarde el fichero de texto con el nombre \"entrada_ORM-4P.txt\" en la misma ubicacin donde se encuentra");
						System.out
								.println("el proyecto y pulse \"C\" para continuar. Pulse \"S\" para abandonar la aplicacin.");
						System.out
								.println("Si desea ms informacin acerca del formato del fichero pulse la tecla \"I\".");
						
						cadenaEntrada = in.readLine();
						
						if ("I".equals(cadenaEntrada)
								|| "i".equals(cadenaEntrada))
						{
							System.out.println(Constantes.INFO_FICHERO_ORM_4P);
						}

						else
						{
							if ("S".equals(cadenaEntrada)
									|| "s".equals(cadenaEntrada))
							{
								System.out
										.println("La aplicacin ha finalizado correctamente.");
								salir = true;
								break;
							} else if ("C".equals(cadenaEntrada)
									|| "c".equals(cadenaEntrada))
							{
								System.out.println("Leyendo el fichero:");
								BufferedReader br = new BufferedReader(
										new FileReader("entrada_ORM-4P.txt"));
								String s;
								if ((s = br.readLine()) != null)
								{
									numElementos = Integer.valueOf(s.trim());
								}
								if (1 > numElementos)
								{
									System.out
											.println("El nmero de elementos debe ser mayor que cero.");
									salir = true;
									break;
								} else
								{
									puntos = new ArrayList<Integer>(
											numElementos);
									alturas = new ArrayList<Integer>(
											numElementos);
									anchuras = new ArrayList<Integer>(
											numElementos);
									i = 2;
									while (((s = br.readLine()) != null)
											&& !error)
									{
										if (i <= (numElementos + 1))
										{
											puntos.add(Integer
													.valueOf(s.trim()));
										} else if (i <= 2 * numElementos + 1)
										{
											alturas.add(Integer.valueOf(s
													.trim()));
										} else if (i <= 3 * numElementos + 1)
										{
											anchuras.add(Integer.valueOf(s
													.trim()));
										} else
										{
											error = true;
										}
										i++;
									}
								}
								br.close();
								finProblema = true;
							}
						}
					}

					else if ("S".equals(cadenaEntrada)
							|| "s".equals(cadenaEntrada))
					{
						System.out
								.println("La aplicacin ha finalizado correctamente.");
						salir = true;
						break;
					}
				}
				
				if (finProblema)
				{
					if (puntos.size() != numElementos
							|| alturas.size() != numElementos
							|| anchuras.size() != numElementos)
					{
						System.out
								.println("Se ha producido un error en la obtencin de los datos: El nmero total de puntos y etiquetas no coincide con el indicado.");
						salir = true;
					}
				}
			}
			
			if (finProblema && !salir)
			{
				System.out
						.println("Las tripletas (punto, altura, anchura) indicadas han sido:");
				for (i = 0; i < numElementos; i++)
				{
					System.out.println("( " + puntos.get(i) + " , "
							+ alturas.get(i) + " , "
							+ anchuras.get(i) + " )");
				}
				solucion = new Solucion_ORM_4P(puntos, alturas, anchuras);
				double tiempoDeEjecucion1 = System.currentTimeMillis();
				solucion.obtenerSolucion();
				double tiempoDeEjecucion2 = System.currentTimeMillis();
				System.out.println("Tiempo de ejecucin: "
						+ (tiempoDeEjecucion2 - tiempoDeEjecucion1) + "ms");
			}
			
		} catch (NumberFormatException e)
		{
			System.out
					.println("\nSe ha producido una excepcin al introducir un valor incorrecto: NumberFormatException.");
			System.out
					.println("Vuelva a ejecutar el programa si desea continuar.");
			e.printStackTrace();
		} catch (IOException e)
		{
			System.out
					.println("\nSe ha producido una excepcin de entrada/salida: IOException.");
			System.out
					.println("Vuelva a ejecutar el programa si desea continuar.");
			e.printStackTrace();
		} catch (ArrayIndexOutOfBoundsException e)
		{
			System.out
					.println("\nSe ha producido un error: ArrayIndexOutOfBoundsException.");
			System.out
					.println("Vuelva a ejecutar el programa si desea continuar.");
			e.printStackTrace();
		}
	}
}
